{{File: ServosObjectExample.spin
This example top file controls three servos with the Serovs object.}}
      
con

  ' Declare clockmode and oscillator frequency.
  ' Use an external oscillator, preferably in some multiple of 1 MHz
  _xinfreq = 5_000_000                        ' 5.000 Mhz oscillaotr
  _clkmode = xtal1 + pll16x                   ' 16x PLL -> 80 MHz

var

  ' Declare 3 servo array variables.
  long servo[3]

obj

  ' Declare servo object and name it (s in this case).
  s : "servos"

pub init | i

  {Initialize all servo pulse widths.  By setting each servo array element
  to 0, it will result in all I/O lines transmitting 1.5 ms when the Servos
  object starts the cog. Variations above or below 0 are in 1 us units.  So,
  for 2 ms pulses, set a given array element to 500, or for 1 ms pulses, set
  it to -500, and so on...}

  servo[0] := 0                              ' Set pulse widths to 1.5 ms
  servo[1] := 0
  servo[2] := 0

  {The servo object assumes a group of servos is connected to contiguous pins.
  The first two arguments of s.start are the start and end pins. After calling
  s.start, servo[0] controls A16 pulse widths, servo[1] controls A17 pulse
  widths, and servo[2] controls A18 pulse widths.

  After starting the Servos object, the cog repeats the servo pulse every 20 ms.

  Any change you make to the values of servo[0], servo[1], or servo[2] will
  update the pulse train transmitted by A16, A17, or A18 respectively.}
  
  s.start(16, 18, @servo)                    ' Start s(ervo) object

  main                                       ' Call main method            

pub main 

  waitcnt(cnt + clkfreq * 3)                 ' 3 servos centered for 3 s
  servo[1] := 500                            ' Set A17 to 2 ms pulses
  waitcnt(cnt + clkfreq * 2)                 ' Hold for 2 s
  BackAndForth                               ' Call back and forth method

pub BackAndForth | drive, incDec, limit

  {Repeatedly ramps the pulse widths of the A16 servo from 2 to 1 ms pulses
  at the same time it ramps A17 from 1 to 2 ms.  The loop holds both the
  2 ms and 1 ms pulse trains for 2 seconds before ramping to the other end.}
  
  drive := 0                                 ' Initialize drive to 0
  incDec := 1                                ' Start incementing by + 1
  limit := 500                               ' Absolute +/- limit

  repeat                                     ' Endless outer loop
    waitcnt(cnt + (clkfreq * 2))             ' Pause before repeat outer loop
    repeat                                   ' Inner loop
      servo[1] := -servo[0] := drive         ' Opposing drives for A16 & 17
      waitcnt(clkfreq / 500 + cnt)           ' Approx 2 ms pause
      drive += incDec                        ' Increment(or decrement) drive
      if || drive == || limit                ' If abs(drive) = abs(limit)
        - limit                              ' Change sign of limit
        - incDec                             ' Change sign of incDec
        quit                                 ' Exit inner loop